@@ -2,21 +2,21 @@
  *
  *    GDMflexiserver - run a flexible server
  *    (c)2001 Queen of England
- *    
+ *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
  *    the Free Software Foundation; either version 2 of the License, or
  *    (at your option) any later version.
- *   
+ *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *    GNU General Public License for more details.
- *   
+ *
  *    You should have received a copy of the GNU General Public License
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- * 
+ *
  */
 
 #include "config.h"
@@ -37,11 +37,18 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
+#include <X11/Xauth.h>
+
 #include "gdm.h"
 #include "gdmcomm.h"
 #include "gdmcommon.h"
 #include "gdmconfig.h"
-#include "gdmconfig.h"
+
+#include "gdm-common.h"
+#include "gdm-log.h"
+#include "gdm-socket-protocol.h"
+#include "gdm-daemon-config-keys.h"
+#include "server.h"
 
 static GSList *xservers          = NULL;
 static const char *send_command  = NULL;
@@ -72,49 +79,82 @@ GOptionEntry options [] = {
 static int
 get_cur_vt (void)
 {
-	char *ret;
-	static int cur_vt;
+	char           *result;
+	static int      cur_vt;
 	static gboolean checked = FALSE;
+	char           *ret = NULL;
 
-	if (checked)
+	result = NULL;
+
+	if (checked) {
 		return cur_vt;
-
-	ret = gdmcomm_call_gdm ("QUERY_VT", auth_cookie, "2.5.90.0", 5);
-	if (ve_string_empty (ret) ||
-	    strncmp (ret, "OK ", 3) != 0) {
-		g_free (ret);
-		return -1;
 	}
 
-	if (sscanf (ret, "OK %d", &cur_vt) != 1)
+	ret = gdmcomm_call_gdm ("QUERY_VT", auth_cookie, "2.5.90.0", 5);
+	if (ve_string_empty (ret) || strncmp (ret, "OK ", 3) != 0) {
+		goto out;
+	}
+
+	if (sscanf (ret, "OK %d", &cur_vt) != 1) {
 		cur_vt = -1;
-	g_free (ret);
+	}
+
 	checked = TRUE;
+
+ out:
+	g_free (ret);
+
 	return cur_vt;
 }
 
+static GtkWidget *
+hig_dialog_new (GtkWindow      *parent,
+		GtkDialogFlags flags,
+		GtkMessageType type,
+		GtkButtonsType buttons,
+		const gchar    *primary_message,
+		const gchar    *secondary_message)
+{
+	GtkWidget *dialog;
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+		                         GTK_DIALOG_DESTROY_WITH_PARENT,
+		                         type,
+		                         buttons,
+		                         "%s", primary_message);
+
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+		                                  "%s", secondary_message);
+
+	gtk_window_set_title (GTK_WINDOW (dialog), "");
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
+
+  	return dialog;
+}
+
 /* change to an existing vt */
 static void
 change_vt (int vt)
 {
 	char *cmd;
 	char *ret;
+
 	cmd = g_strdup_printf (GDM_SUP_SET_VT " %d", vt);
 	ret = gdmcomm_call_gdm (cmd, auth_cookie, "2.5.90.0", 5);
 	g_free (cmd);
+
 	if (ve_string_empty (ret) ||
 	    strcmp (ret, "OK") != 0) {
 		GtkWidget *dialog;
 		const char *message = gdmcomm_get_error_message (ret, use_xnest);
 
-		dialog = ve_hig_dialog_new
-			(NULL /* parent */,
-			 GTK_DIALOG_MODAL /* flags */,
-			 GTK_MESSAGE_ERROR,
-			 GTK_BUTTONS_OK,
-			 _("Cannot change display"),
-			 message);
-
+		dialog = hig_dialog_new (NULL /* parent */,
+					 GTK_DIALOG_MODAL /* flags */,
+					 GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_OK,
+					 _("Cannot change display"),
+					 message);
 		gtk_widget_show_all (dialog);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
@@ -136,9 +176,10 @@ get_vt_num (char **vec, char *vtpart, int depth)
 	for (i = 0; vec[i] != NULL; i++) {
 		char **rvec;
 		rvec = g_strsplit (vec[i], ",", -1);
-		if (rvec == NULL ||
-		    ve_vector_len (rvec) != 3)
+		if (gdm_vector_len (rvec) != 3) {
+			g_strfreev (rvec);
 			continue;
+		}
 
 		if (strcmp (rvec[0], vtpart) == 0) {
 			/* could be nested? */
@@ -177,9 +218,10 @@ create_model (char **vec)
 		char **rvec;
 		int vt;
 		rvec = g_strsplit (vec[i], ",", -1);
-		if (rvec == NULL ||
-		    ve_vector_len (rvec) != 3)
+		if (gdm_vector_len (rvec) != 3) {
+			g_strfreev (rvec);
 			continue;
+		}
 
 		vt = get_vt_num (vec, rvec[2], 5);
 
@@ -350,18 +392,15 @@ maybe_lock_screen (void)
 
 	g_free (command);
 
-	if (use_gscreensaver) {
-		command = g_strdup ("gnome-screensaver-command --throttle");
-	} else {
+	if (! use_gscreensaver) {
 		command = g_strdup ("xscreensaver-command -throttle");
-	}
+		if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
+			g_warning ("Cannot disable screensaver engines: %s", error->message);
+			g_error_free (error);
+		}
 
-	if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
-		g_warning ("Cannot disable screensaver engines: %s", error->message);
-		g_error_free (error);
+		g_free (command);
 	}
-
-	g_free (command);
 }
 
 static void
@@ -380,7 +419,7 @@ run_logged_in_dialogue (char **vec)
 	if (startnew == TRUE) {
 		/* Just return if the user doesn't want to see the dialog */
 		return;
-	} 
+	}
 
 	dialog = gtk_dialog_new_with_buttons (_("Open Displays"),
 					      NULL /* parent */,
@@ -516,9 +555,10 @@ check_for_users (void)
 		char **rvec;
 		int vt;
 		rvec = g_strsplit (vec[i], ",", -1);
-		if (rvec == NULL ||
-		    ve_vector_len (rvec) != 3)
+		if (gdm_vector_len (rvec) != 3) {
+			g_strfreev (rvec);
 			continue;
+		}
 
 		vt = get_vt_num (vec, rvec[2], 5);
 
@@ -705,45 +745,43 @@ main (int argc, char *argv[])
 	char *version;
 	char *ret;
 	const char *message;
-	GOptionContext *ctx;
+	gboolean    initialized;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	/* Option parsing */
-	ctx = g_option_context_new("- New gdm login");
-        g_option_context_add_main_entries(ctx, options, _("main options"));
-        g_option_context_parse(ctx, &argc, &argv, NULL);
-        g_option_context_free(ctx);
+	initialized = gtk_init_with_args (&argc,
+					  &argv,
+					  "- New gdm login",
+					  options,
+					  GETTEXT_PACKAGE,
+					  NULL);
 
 	if (monte_carlo_pi) {
 		calc_pi ();
 		return 0;
 	}
 
-	gdmcomm_set_debug (debug_in);
+	gdm_log_init ();
+	gdm_log_set_debug (debug_in);
 
 	if (args_remaining != NULL && args_remaining[0] != NULL)
 		server = args_remaining[0];
 
 	if (send_command != NULL) {
 		if ( ! gdmcomm_check (FALSE)) {
-			gdm_common_error ("Error: GDM (GNOME Display Manager) "
-			"is not running.");
-			gdm_common_error ("You might be using a different "
-			"display manager.");
+			gdm_common_error (_("Error: GDM (GNOME Display Manager) is not running."));
+			gdm_common_error (_("You might be using a different display manager."));
 			return 1;
 		}
 	} else {
-		/*
-		 * The --command argument does not display anything, so avoid
-		 * running gtk_init until it finishes.  Sometimes the
-		 * --command argument is used when there is no display so it
-		 * will fail and cause the program to exit, complaining about
-		 * "no display".
-		 */
-		gtk_init(&argc, &argv);
+
+		if (! initialized) {
+			g_warning ("Unable to open a display");
+			exit (1);
+		}
+
 		if ( ! gdmcomm_check (TRUE)) {
 			return 1;
 		}
@@ -753,9 +791,6 @@ main (int argc, char *argv[])
 	gdmcomm_comm_bulk_start ();
 
 	/* Process --command option */
-
-        g_type_init ();
-
 	if (send_command != NULL) {
 		if (authenticate)
 			auth_cookie = gdmcomm_get_auth_cookie ();
@@ -797,15 +832,14 @@ main (int argc, char *argv[])
 			g_print ("%s\n", ret);
 			return 0;
 		} else {
-			dialog = ve_hig_dialog_new
-				(NULL /* parent */,
-				 GTK_DIALOG_MODAL /* flags */,
-				 GTK_MESSAGE_ERROR,
-				 GTK_BUTTONS_OK,
-				 _("Cannot communicate with GDM "
-				   "(The GNOME Display Manager)"),
-				 _("Perhaps you have an old version "
-				   "of GDM running."));
+			dialog = hig_dialog_new (NULL /* parent */,
+						 GTK_DIALOG_MODAL /* flags */,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 _("Cannot communicate with GDM "
+						   "(The GNOME Display Manager)"),
+						 _("Perhaps you have an old version "
+						   "of GDM running."));
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
@@ -832,9 +866,6 @@ main (int argc, char *argv[])
 	 */
 	auth_cookie = gdmcomm_get_auth_cookie ();
 
-	/* check for other displays/logged in users */
-	check_for_users ();
-
 	if (use_xnest) {
 		char *cookie = gdmcomm_get_a_cookie (FALSE /* binary */);
 
@@ -843,17 +874,15 @@ main (int argc, char *argv[])
 			/* At this point we are done using the socket, so close it */
 			gdmcomm_comm_bulk_stop ();
 
-			dialog = ve_hig_dialog_new
-				(NULL /* parent */,
-				 GTK_DIALOG_MODAL /* flags */,
-				 GTK_MESSAGE_ERROR,
-				 GTK_BUTTONS_OK,
-				 _("You do not seem to have the "
-				   "authentication needed for this "
-				   "operation"),
-				 _("Perhaps your .Xauthority "
-				   "file is not set up correctly."));
-
+			dialog = hig_dialog_new (NULL /* parent */,
+						 GTK_DIALOG_MODAL /* flags */,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 _("You do not seem to have the "
+						   "authentication needed for this "
+						   "operation"),
+						 _("Perhaps your .Xauthority "
+						   "file is not set up correctly."));
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
@@ -868,20 +897,23 @@ main (int argc, char *argv[])
 		version = "2.3.90.4";
 		auth_cookie = NULL;
 	} else {
+
+		/* check for other displays/logged in users */
+		check_for_users ();
+
 		if (auth_cookie == NULL) {
 
 			/* At this point we are done using the socket, so close it */
 			gdmcomm_comm_bulk_stop ();
 
-			dialog = ve_hig_dialog_new
-				(NULL /* parent */,
-				 GTK_DIALOG_MODAL /* flags */,
-				 GTK_MESSAGE_ERROR,
-				 GTK_BUTTONS_OK,
-				 _("You do not seem to be logged in on the "
-				   "console"),
-				 _("Starting a new login only "
-				   "works correctly on the console."));
+			dialog = hig_dialog_new (NULL /* parent */,
+						 GTK_DIALOG_MODAL /* flags */,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 _("You do not seem to be logged in on the "
+						   "console"),
+						 _("Starting a new login only "
+						   "works correctly on the console."));
 			gtk_dialog_set_has_separator (GTK_DIALOG (dialog),
 						      FALSE);
 			gtk_widget_show_all (dialog);
@@ -923,13 +955,12 @@ main (int argc, char *argv[])
 
 	message = gdmcomm_get_error_message (ret, use_xnest);
 
-	dialog = ve_hig_dialog_new
-		(NULL /* parent */,
-		 GTK_DIALOG_MODAL /* flags */,
-		 GTK_MESSAGE_ERROR,
-		 GTK_BUTTONS_OK,
-		 _("Cannot start new display"),
-		 message);
+	dialog = hig_dialog_new (NULL /* parent */,
+				 GTK_DIALOG_MODAL /* flags */,
+				 GTK_MESSAGE_ERROR,
+				 GTK_BUTTONS_OK,
+				 _("Cannot start new display"),
+				 message);
 
 	gtk_widget_show_all (dialog);
 	gtk_dialog_run (GTK_DIALOG (dialog));
