@@ -1,4 +1,6 @@
-/* GDM - The GNOME Display Manager
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
+ * GDM - The GNOME Display Manager
  * Copyright (C) 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This file Copyright (c) 2003 George Lebl
@@ -21,18 +23,20 @@
 
 #include "config.h"
 
+#include <stdlib.h>
 #include <unistd.h>
+#include <string.h>
 #include <dirent.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include "vicious.h"
-
 #include "gdm.h"
 #include "gdmsession.h"
 #include "gdmcommon.h"
 #include "gdmconfig.h"
-#include "gdmwm.h"
+
+#include "gdm-common.h"
+#include "gdm-daemon-config-keys.h"
 
 GHashTable *sessnames        = NULL;
 gchar *default_session       = NULL;
@@ -107,9 +111,21 @@ gdm_session_list_from_hash_table_func (const char *key, const char *value,
         *sessions = g_list_prepend (*sessions, g_strdup (key));
 }
 
+/* Just a wrapper to ensure compatibility with the
+   existing code */
 void
 gdm_session_list_init ()
 {
+	_gdm_session_list_init (&sessnames, &sessions, 
+				&default_session, &current_session);	
+}
+
+/* The real gdm_session_list_init */
+void
+_gdm_session_list_init (GHashTable **sessnames, GList **sessions, 
+			gchar **default_session, const gchar **current_session)
+{
+
     GdmSession *session = NULL;
     gboolean some_dir_exists = FALSE;
     gboolean searching_for_default = TRUE;
@@ -119,7 +135,7 @@ gdm_session_list_init ()
     DIR *sessdir;
     int i;
 
-    sessnames = g_hash_table_new (g_str_hash, g_str_equal);
+    *sessnames = g_hash_table_new (g_str_hash, g_str_equal);
 
     if (gdm_config_get_bool (GDM_KEY_SHOW_GNOME_FAILSAFE)) {
 	session = g_new0 (GdmSession, 1);
@@ -130,7 +146,7 @@ gdm_session_list_init ()
 		"and it is only to be used when you can't log "
 		"in otherwise.  GNOME will use the 'Default' "
 		"session."));
-	g_hash_table_insert (sessnames, g_strdup (GDM_SESSION_FAILSAFE_GNOME), session);
+	g_hash_table_insert (*sessnames, g_strdup (GDM_SESSION_FAILSAFE_GNOME), session);
     }
 
     if (gdm_config_get_bool (GDM_KEY_SHOW_XTERM_FAILSAFE)) {
@@ -143,7 +159,7 @@ gdm_session_list_init ()
 		"and it is only to be used when you can't log "
 		"in otherwise.  To exit the terminal, "
 		"type 'exit'."));
-	g_hash_table_insert (sessnames, g_strdup (GDM_SESSION_FAILSAFE_XTERM),
+	g_hash_table_insert (*sessnames, g_strdup (GDM_SESSION_FAILSAFE_XTERM),
 		session);
     }
 
@@ -167,12 +183,13 @@ gdm_session_list_init ()
 		    dent = NULL;
 
 	    while (dent != NULL) {
-		    VeConfig *cfg;
+		    GKeyFile *cfg;
 		    char *exec;
 		    char *comment;
 		    char *s;
 		    char *tryexec;
 		    char *ext;
+		    gboolean hidden;
 
 		    /* ignore everything but the .desktop files */
 		    ext = strstr (dent->d_name, ".desktop");
@@ -183,54 +200,66 @@ gdm_session_list_init ()
 		    }
 
 		    /* already found this session, ignore */
-		    if (g_hash_table_lookup (sessnames, dent->d_name) != NULL) {
+		    if (g_hash_table_lookup (*sessnames, dent->d_name) != NULL) {
 			    dent = readdir (sessdir);
 			    continue;
 		    }
 
 		    s = g_strconcat (dir, "/", dent->d_name, NULL);
-		    cfg = ve_config_new (s);
+		    cfg = gdm_common_config_load (s, NULL);
 		    g_free (s);
 
-		    if (ve_config_get_bool (cfg, "Desktop Entry/Hidden=false")) {
+		    hidden = FALSE;
+		    gdm_common_config_get_boolean (cfg, "Desktop Entry/Hidden=false", &hidden, NULL);
+		    if (hidden) {
 			    session = g_new0 (GdmSession, 1);
 			    session->name      = g_strdup (dent->d_name);
 			    session->clearname = NULL;
-			    g_hash_table_insert (sessnames, g_strdup (dent->d_name), session);
-			    ve_config_destroy (cfg);
+			    g_hash_table_insert (*sessnames, g_strdup (dent->d_name), session);
+			    g_key_file_free (cfg);
 			    dent = readdir (sessdir);
 			    continue;
 		    }
 
-		    tryexec = ve_config_get_string (cfg, "Desktop Entry/TryExec");
+		    tryexec = NULL;
+		    gdm_common_config_get_string (cfg, "Desktop Entry/TryExec", &tryexec, NULL);
 		    if ( ! ve_string_empty (tryexec)) {
-			    char *full = g_find_program_in_path (tryexec);
+			    char **tryexecvec = g_strsplit (tryexec, " ", -1);
+			    char *full = NULL;
+
+			    /* Do not pass any arguments to g_find_program_in_path */
+			    if (tryexecvec != NULL)
+				full = g_find_program_in_path (tryexecvec[0]);
+
 			    if (full == NULL) {
 				    session = g_new0 (GdmSession, 1);
 				    session->name      = g_strdup (dent->d_name);
 				    session->clearname = NULL;
-				    g_hash_table_insert (sessnames, g_strdup (dent->d_name),
+				    g_hash_table_insert (*sessnames, g_strdup (dent->d_name),
 					 session);
 				    g_free (tryexec);
-				    ve_config_destroy (cfg);
+				    g_key_file_free (cfg);
 				    dent = readdir (sessdir);
 				    continue;
 			    }
+			    g_strfreev (tryexecvec);
 			    g_free (full);
 		    }
 		    g_free (tryexec);
 
-		    exec = ve_config_get_string (cfg, "Desktop Entry/Exec");
-		    name = ve_config_get_translated_string (cfg, "Desktop Entry/Name");
-		    comment = ve_config_get_translated_string (cfg, "Desktop Entry/Comment");
-
-		    ve_config_destroy (cfg);
+		    exec = NULL;
+		    name = NULL;
+		    comment = NULL;
+		    gdm_common_config_get_string (cfg, "Desktop Entry/Exec", &exec, NULL);
+		    gdm_common_config_get_translated_string (cfg, "Desktop Entry/Name", &name, NULL);
+		    gdm_common_config_get_translated_string (cfg, "Desktop Entry/Comment", &comment, NULL);
+		    g_key_file_free (cfg);
 
 		    if G_UNLIKELY (ve_string_empty (exec) || ve_string_empty (name)) {
 			    session = g_new0 (GdmSession, 1);
 			    session->name      = g_strdup (dent->d_name);
 			    session->clearname = NULL;
-			    g_hash_table_insert (sessnames, g_strdup (dent->d_name), session);
+			    g_hash_table_insert (*sessnames, g_strdup (dent->d_name), session);
 			    g_free (exec);
 			    g_free (name);
 			    g_free (comment);
@@ -239,34 +268,36 @@ gdm_session_list_init ()
 		    }
 
 		    /* if we found the default session */
-		    if ( ! ve_string_empty (gdm_config_get_string (GDM_KEY_DEFAULT_SESSION)) &&
-			 strcmp (dent->d_name, gdm_config_get_string (GDM_KEY_DEFAULT_SESSION)) == 0) {
-			    g_free (default_session);
-			    default_session = g_strdup (dent->d_name);
-			    searching_for_default = FALSE;
-		    }
+		    if (default_session != NULL) {
+			    if ( ! ve_string_empty (gdm_config_get_string (GDM_KEY_DEFAULT_SESSION)) &&
+				 strcmp (dent->d_name, gdm_config_get_string (GDM_KEY_DEFAULT_SESSION)) == 0) {
+				    g_free (*default_session);
+				    *default_session = g_strdup (dent->d_name);
+				    searching_for_default = FALSE;
+			    }		    
 
-		    /* if there is a session called Default */
-		    if (searching_for_default &&
-			g_ascii_strcasecmp (dent->d_name, "default.desktop") == 0) {
-			    g_free (default_session);
-			    default_session = g_strdup (dent->d_name);
-		    }
+			    /* if there is a session called Default */
+			    if (searching_for_default &&
+				g_ascii_strcasecmp (dent->d_name, "default.desktop") == 0) {
+				    g_free (*default_session);
+				    *default_session = g_strdup (dent->d_name);
+			    }
 
-		    if (searching_for_default &&
-			g_ascii_strcasecmp (dent->d_name, "gnome.desktop") == 0) {
-			    /* Just in case there is no default session and
-			     * no default link, make gnome the default */
-			    if (default_session == NULL)
-				    default_session = g_strdup (dent->d_name);
+			    if (searching_for_default &&
+				g_ascii_strcasecmp (dent->d_name, "gnome.desktop") == 0) {
+				    /* Just in case there is no default session and
+				     * no default link, make gnome the default */
+				    if (*default_session == NULL)
+					    *default_session = g_strdup (dent->d_name);
 
+			    }
 		    }
 
 		    session = g_new0 (GdmSession, 1);
 		    session->name      = g_strdup (name);
 		    session->clearname = NULL;
 		    session->comment   = g_strdup (comment);
-		    g_hash_table_insert (sessnames, g_strdup (dent->d_name), session);
+		    g_hash_table_insert (*sessnames, g_strdup (dent->d_name), session);
 		    g_free (exec);
 		    g_free (comment);
 		    dent = readdir (sessdir);
@@ -286,12 +317,13 @@ gdm_session_list_init ()
 	session_dir_whacked_out = TRUE;
     }
 
-    if G_UNLIKELY (g_hash_table_size (sessnames) == 0) {
+    if G_UNLIKELY (g_hash_table_size (*sessnames) == 0) {
 	    gdm_common_warning ("Error, no sessions found in the session directory <%s>.",
 		ve_sure_string (gdm_config_get_string (GDM_KEY_SESSION_DESKTOP_DIR)));
 
 	    session_dir_whacked_out = TRUE;
-	    default_session         = g_strdup (GDM_SESSION_FAILSAFE_GNOME);
+	    if (default_session != NULL)
+		    *default_session = g_strdup (GDM_SESSION_FAILSAFE_GNOME);
     }
 
 
@@ -304,7 +336,7 @@ gdm_session_list_init ()
                                     "and it is only to be used when you can't log "
                                     "in otherwise.  GNOME will use the 'Default' "
 				    "session."));
-	    g_hash_table_insert (sessnames,
+	    g_hash_table_insert (*sessnames,
 		g_strdup (GDM_SESSION_FAILSAFE_GNOME), session);
     }
 
@@ -317,24 +349,28 @@ gdm_session_list_init ()
 				    "and it is only to be used when you can't log "
 				    "in otherwise.  To exit the terminal, "
 				    "type 'exit'."));
-	    g_hash_table_insert (sessnames,
+	    g_hash_table_insert (*sessnames,
 		g_strdup (GDM_SESSION_FAILSAFE_XTERM), session);
     }
 
     /* Convert to list (which is unsorted) */
-    g_hash_table_foreach (sessnames,
-	(GHFunc) gdm_session_list_from_hash_table_func, &sessions);
+    g_hash_table_foreach (*sessnames,
+	(GHFunc) gdm_session_list_from_hash_table_func, sessions);
 
     /* Prioritize and sort the list */
-    sessions = g_list_sort (sessions, (GCompareFunc) gdm_session_sort_func);
+    *sessions = g_list_sort (*sessions, (GCompareFunc) gdm_session_sort_func);
 
-    if G_UNLIKELY (default_session == NULL) {
-	    default_session = g_strdup (GDM_SESSION_FAILSAFE_GNOME);
-	    gdm_common_warning ("No default session link found. Using Failsafe GNOME.");
-    }
+    if (default_session != NULL)
+	    if G_UNLIKELY (*default_session == NULL) {
+		    *default_session = g_strdup (GDM_SESSION_FAILSAFE_GNOME);
+		    gdm_common_warning ("No default session link found. Using Failsafe GNOME.");
+	    }
     
-    if (current_session == NULL)
-            current_session = default_session;
+    if (current_session != NULL &&
+	default_session != NULL) {
+	    if (*current_session == NULL)
+		    *current_session = *default_session;
+    }
 }
 
 static gboolean
@@ -364,10 +400,13 @@ gdm_login_list_lookup (GList *l, const gchar *data)
 }
 
 char *
-gdm_session_lookup (const char *saved_session)
+gdm_session_lookup (const char *saved_session, gint *lookup_status)
 {
   gchar *session = NULL;
   
+  /* Assume that the lookup will go well */
+  *lookup_status = SESSION_LOOKUP_SUCCESS;
+
   /* Don't save session unless told otherwise */
   save_session = GTK_RESPONSE_NO;
 
@@ -395,21 +434,10 @@ gdm_session_lookup (const char *saved_session)
       /* Check if user's saved session exists on this box */
       if (!gdm_login_list_lookup (sessions, session))
 	{
-	  gchar *firstmsg;
-	  gchar *secondmsg;
-	  
+		
           g_free (session);
 	  session = g_strdup (default_session);
-	  firstmsg = g_strdup_printf (_("Do you wish to make %s the default for "
-	                                "future sessions?"),
-	                              gdm_session_name (saved_session));	    
-	  secondmsg = g_strdup_printf (_("Your preferred session type %s is not "
-	                                 "installed on this computer."),
-	                               gdm_session_name (default_session));
-	  save_session = gdm_wm_query_dialog (firstmsg, secondmsg,
-		_("Make _Default"), _("Just _Log In"), TRUE);
-	  g_free (firstmsg);
-	  g_free (secondmsg);
+	  *lookup_status = SESSION_LOOKUP_PREFERRED_MISSING;
 	}
     }
   else /* One of the other available session types is selected */
@@ -430,22 +458,10 @@ gdm_session_lookup (const char *saved_session)
 	  save_session = GTK_RESPONSE_NO;
 	}
       else if (strcmp (saved_session, session) != 0)
-	{
-	  gchar *firstmsg = NULL;
-	  gchar *secondmsg = NULL;
-	  
+	{	 	  
 	  if (gdm_config_get_bool (GDM_KEY_SHOW_LAST_SESSION))
 	    {
-	      firstmsg = g_strdup_printf (_("Do you wish to make %s the default for "
-	                                    "future sessions?"),
-	                                  gdm_session_name (session));
-	      secondmsg = g_strdup_printf (_("You have chosen %s for this "
-	                                     "session, but your default "
-	                                     "setting is %s."),
-	                                   gdm_session_name (session),
-	                                   gdm_session_name (saved_session));
-	      save_session = gdm_wm_query_dialog (firstmsg, secondmsg,
-			_("Make _Default"), _("Just For _This Session"), TRUE);
+		    *lookup_status = SESSION_LOOKUP_DEFAULT_MISMATCH;
 	    }
 	  else if (strcmp (session, default_session) != 0 &&
 		   strcmp (session, saved_session) != 0 &&
@@ -458,21 +474,10 @@ gdm_session_lookup (const char *saved_session)
 	       */
 	      if (g_access ("/usr/bin/switchdesk", F_OK) == 0)
 	        {
-	          firstmsg = g_strdup_printf (_("You have chosen %s for this "
-	                                        "session"),
-	                                      gdm_session_name (session));
-	          secondmsg = g_strdup_printf (_("If you wish to make %s "
-	                                         "the default for future sessions, "
-	                                         "run the 'switchdesk' utility "
-	                                         "(System->Desktop Switching Tool from "
-	                                         "the panel menu)."),
-	                                       gdm_session_name (session));			 
-		  gdm_wm_message_dialog (firstmsg, secondmsg);
+			*lookup_status = SESSION_LOOKUP_USE_SWITCHDESK;
 		}
 	      save_session = GTK_RESPONSE_NO;
 	    }
-	  g_free (firstmsg);
-	  g_free (secondmsg);
 	}
     }
 
@@ -485,3 +490,14 @@ gdm_get_save_session (void)
   return save_session;
 }
 
+void
+gdm_set_save_session (const gint session)
+{
+	save_session = session;
+}
+
+const char*
+gdm_get_default_session (void)
+{
+	return default_session;
+}
